package labtwo;

import java.awt.*;
import java.awt.geom.Point2D;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: zaic
 * Date: 4/11/12
 * Time: 10:07 PM
 */
public class LemniscateBresenhamDrawer implements LemniscateDrawer{
    static final int dir[][] = { {1, 1}, {1, 0}, {-1, 0}, {0, 1}, {-1, 1} };
    static final int coords[][] = { {1, 1}, {-1, 1}, {1, -1}, {-1, -1} };
    double c = 0;
    double cc;

    double calcError(double x, double y) {
        double reqy = Math.sqrt(cc * cc + 4 * x * x * cc);
        reqy = reqy - x * x - cc;
        if(reqy < 0) reqy = 0;
        reqy = Math.sqrt(reqy);
        return Math.abs(Math.abs(y) - reqy);
    }

    @Override
    public void draw(PlotPanel p, Graphics2D g, Point2D.Double focus_a, Point2D.Double focus_b) {
        c = focus_a.distance(focus_b) / 2.0;
        cc = c * c;
        double a = c * Math.sqrt(2);

        double startx = - Math.sqrt(2) * c;
        int x = (int)(startx * p.getAxesStep() - .5);
        int y = 0;
        int prevx = x, prevy = y;

        int bestdir = 3;
        ArrayList<Point2D.Double> points[] = (ArrayList<Point2D.Double>[])new ArrayList[4];
        for(int i = 0; i < points.length; i++) points[i] = new ArrayList<Point2D.Double>();
        for(; x != 1; ) {
            for(int i = 0; i < 4; i++)
                points[i].add(new Point2D.Double(x * coords[i][1], y * coords[i][0]));

            double best_error = Double.NaN;
            for(int i = 0; i < dir.length; i++) {
                int nexty = y + dir[i][0];
                int nextx = x + dir[i][1];
                if(nextx != prevx || nexty != prevy) {
                    double curr_error = calcError((double)nextx / p.getAxesStep(), (double)nexty / p.getAxesStep());
                    if(Double.isNaN(best_error) || curr_error < best_error) {
                        best_error = curr_error;
                        bestdir = i;
                    }
                }
            }

            prevx = x;
            prevy = y;

            y += dir[bestdir][0];
            x += dir[bestdir][1];
        }

        Point2D.Double allpoints[] = new Point2D.Double[points[0].size() * 4 + 1];
        int last = 0;
        for(int i = 0; i < points[0].size(); i++) allpoints[last++] = points[0].get(i);
        for(int i = 0; i < points[2].size(); i++) allpoints[last++] = points[2].get(points[2].size() - i - 1);
        for(int i = 0; i < points[3].size(); i++) allpoints[last++] = points[3].get(i);
        for(int i = 0; i < points[1].size(); i++) allpoints[last++] = points[1].get(points[1].size() - i - 1);
        allpoints[last] = points[0].get(0);
        
        Color prev_color = g.getColor();
        g.setColor(Color.GREEN);
        double angle = Math.atan2(focus_a.y - focus_b.y, focus_a.x - focus_b.x);
        double precos = Math.cos(angle);
        double presin = Math.sin(angle);
        for(int i = 0; i < last; i++) {
            int prev_x = (int)(allpoints[i].x * precos - allpoints[i].y * presin);
            int prev_y = (int)(allpoints[i].x * presin + allpoints[i].y * precos);
            int next_x = (int)(allpoints[i + 1].x * precos - allpoints[i + 1].y * presin);
            int next_y = (int)(allpoints[i + 1].x * presin + allpoints[i + 1].y * precos);
            p.drawRealLine(g, prev_x, prev_y, next_x, next_y, false);
        }
        g.setColor(prev_color);
    }
}
