package labtwo;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

/**
 * Created by IntelliJ IDEA.
 * User: zaic
 * Date: 4/4/12
 * Time: 6:54 PM
 */
class PlotPanel extends JPanel {
    Point lineXStart = new Point(0, 0);
    Point lineXEnd = new Point(0, 0);
    Point lineYStart = new Point(0, 0);
    Point lineYEnd = new Point(0, 0);
    Line2D lineX = new Line2D.Double(lineXStart, lineXEnd);
    Line2D lineY = new Line2D.Double(lineYStart, lineYEnd);
    private Stroke stroke = new BasicStroke(2.0f);
    private LineListener lis = null;
    private ScaleChanger scaleListener = null;
    private int axesStep = 40;

    Point2D.Double lemniscate_points[] = {null, null};

    PlotPanel() {
        super();
        addMouseListener(lis = new LineListener(this, lineX));
        addMouseWheelListener(scaleListener = new ScaleChanger());
    }
    
    void setAxesStep(int value) {
        axesStep = Math.max(1, value);
    }

    int getAxesStep() {
        return axesStep;
    }

    double plotToRealX(int x) {
        return ((double)(x - getWidth() / 2)) / axesStep;
    }

    double plotToRealY(int y) {
        return ((double)(getHeight() / 2 - y)) / axesStep;
    }

    /*
    @Override
    public Dimension getPreferredSize(){
        return new Dimension(800, 800);
    }
    */

    /*
           pixel
    */
    void drawRealPixel(Graphics2D g, int x, int y) {
        int realX = x + getWidth() / 2;
        int realY = getHeight() / 2 - y;
        //System.err.println("real pixel: (" + x + ", " + y + ")");
        if(0 <= realX && realX <= getWidth() && 0 <= realY && realY <= getHeight()) {
            g.drawLine(realX, realY, realX, realY);
        }
    }

    void drawPixel(Graphics2D g, double x, double y) {
        int real_x = (int)(x * axesStep);
        int real_y = (int)(y * axesStep);
        drawRealPixel(g, real_x, real_y);
    }

    void drawPixel(Graphics2D g, double x, double y, Color color) {
        Color prev_color = g.getColor();
        g.setColor(color);
        drawPixel(g, x, y);
        g.setColor(prev_color);
    }

    /*
           line
    */
    void drawRealLine(Graphics2D g, int xfrom, int yfrom, int xto, int yto, boolean inverted) {
        //System.err.println("(" + xfrom + ", " + yfrom + ") - (" + xto + ", " + yto + ")");
        int dx = Math.abs(xto - xfrom);
        int dy = Math.abs(yto - yfrom);

        if(dy > dx) {
            drawRealLine(g, -yfrom, xfrom, -yto, xto, !inverted);
            return ;
        }

        if(xto < xfrom) {
            drawRealLine(g, xto, yto, xfrom, yfrom, inverted);
            return ;
        }

        int direction = (yto > yfrom ? 1 : -1);
        for(int i = Math.max(xfrom, -getWidth()); i <= Math.min(xto, getWidth()); i++) {
            double currentY = (i - xfrom) / (double)dx * dy;
            int add = (int)(currentY + .5);
            drawRealPixel(g, inverted ? yfrom + add * direction : i, inverted ? -i : yfrom + add * direction);
        }
    }

    void drawLine(Graphics2D g, double xfrom, double yfrom, double xto, double yto) {
        drawRealLine(g, (int)(xfrom * axesStep), (int)(yfrom * axesStep),
                (int)(xto * axesStep), (int)(yto * axesStep), false);
    }

    void drawCircle(Graphics2D g) {
        final double r = 3;
        final double shiftx = 3;
        final double shifty = 2;
        for(int i = 0; i < 360; i += 10) {
            //System.err.println(i);
            double x = r * Math.cos(i * Math.PI / 180);
            double y = r * Math.sin(i * Math.PI / 180);
            drawLine(g, shiftx, shifty, (shiftx + x), (shifty + y));
        }
    }

    /*
           axes
    */
    void drawAxes(Graphics2D g2) {
        int midWidth = getWidth() / 2;
        int midHeight = getHeight() / 2;

        g2.draw(new Line2D.Double(0, midHeight, getWidth(), midHeight));
        g2.draw(new Line2D.Double(midWidth, 0, midWidth, getHeight()));

        int axesPlotStep = axesStep;
        int axesPrecision = 0;

        for(int i = 0; i <= midWidth; i += axesPlotStep) {
            g2.draw(new Line2D.Double(midWidth + i, midHeight + 3, midWidth + i, midHeight - 3));

            String number = new Integer(i / axesStep).toString();
            g2.drawString(number, midWidth + i - 5, midHeight + 20);

            if(i > 0) {
                g2.draw(new Line2D.Double(midWidth - i, midHeight + 3, midWidth - i, midHeight - 3));
                g2.drawString(new Integer(-i / axesStep).toString(), midWidth - i - 5, midHeight + 20);
            }
        }

        for(int i = 0; i <= midHeight; i += axesPlotStep) {
            g2.draw(new Line2D.Double(midWidth - 3, midHeight - i, midWidth + 3, midHeight - i));
            g2.drawString(new Integer(i / axesStep).toString(), midWidth - 25, midHeight + 5 - i);

            if(i > 0) {
                g2.draw(new Line2D.Double(midWidth - 3, midHeight + i, midWidth + 3, midHeight + i));
                g2.drawString(new Integer(-i / axesStep).toString(), midWidth - 25, midHeight + 5 + i);
            }
        }
    }

    /*
           functions
    */
    /*
    void drawPlot(Graphics2D g, FunctionInterface function) {
        Color prev_color = g.getColor();
        g.setColor(Color.red);
        double prevX = 0.0;
        double prevY = Double.NaN;
        for(int i = 0; i <= 800; i++) {
            double x = ((double)(i - 400)) / axesStep;
            double y = function.f(x);
            if(!Double.isNaN(y) && !Double.isInfinite(y)) {
                drawPixel(g, x, y);
                if(!Double.isNaN(prevY) && !Double.isInfinite(prevY)) {
                    drawLine(g, prevX, prevY, x, y);
                }
            }
            prevX = x;
            prevY = y;
        }
        g.setColor(prev_color);
    }
    */

    void drawx2p5ye1(Graphics2D g) {
        double prevX = Double.NaN;
        double prevUpY = Double.NaN, prevDownY = Double.NaN;
        for(int i = 0; i <= 800; i++) {
            double x = ((double)(i - 400)) / axesStep;
            double y = (1 - x*x*x)/5.0;
            if(y < 0.0) continue;
            double upy = Math.sqrt(y);
            double downy = -upy;
            if(Double.isNaN(prevUpY)) {
                prevUpY = prevDownY = (upy + downy)/2.0;
            }
            drawPixel(g, x, upy);
            drawPixel(g, x, downy);
            //drawLine(g, prevX, prevUpY, x, upy);
            //drawLine(g, prevX, prevDownY, x, downy);
            prevX = x;
            prevUpY = upy;
            prevDownY = downy;
        }
    }

    void drawFunc(Graphics2D g) {
        for(int i = 0; i <= 800; i++)
            for(int j = 0; j <= 800; j++) {
                double x = ((double)(i - 400)) / axesStep;
                double y = ((double)(j - 400)) / axesStep;
                double value = x*x*x + 5.0*y*y - 1.0;
                if(Math.abs(value) < 0.001) {
                    drawPixel(g, x, y);
                }
            }
    }

    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        setBackground(Color.white);
        Graphics2D g2 = (Graphics2D) g;
        g2.setStroke(stroke);
        if (lis.deleteLine){
            g2.setColor(getBackground());
            lis.deleteLine = false;
        }
        else
            g2.setColor(Color.black);
        g2.draw(lineX);
        g2.draw(lineY);

        drawAxes(g2);
        //drawx2p5ye1(g2);
        if(lemniscate_points[0] != null && lemniscate_points[1] != null) {
            if(Window.chk_lemniscate_param.isSelected()) {
                LemniscateParamDrawer drawer = new LemniscateParamDrawer();
                drawer.draw(this, g2, lemniscate_points[0], lemniscate_points[1]);
            }
            if(Window.chk_lemniscate_brez.isSelected()) {
                LemniscateBresenhamDrawer drawer = new LemniscateBresenhamDrawer();
                drawer.draw(this, g2, lemniscate_points[0], lemniscate_points[1]);
            }
        }
    }
    
    /*
     *  LEMNISCATES
     */
    
    void setLemnisactePoint(int x, int y) {
        if(lemniscate_points[0] != null && lemniscate_points[1] != null) {
            lemniscate_points[0] = lemniscate_points[1] = null;
        }
        for(int i = 0; i < 2; i++)
            if(lemniscate_points[i] == null) {
                lemniscate_points[i] = new Point2D.Double(plotToRealX(x), plotToRealY(y));
                break;
            }
        this.repaint();
    }

}
